@@ -24,7 +24,7 @@
  */
 
 #include "includes.h"
-RCSID("$Id: atomicio.c,v 1.1 1999/12/06 20:15:25 deraadt Exp $");
+RCSID("$Id: atomicio.c,v 1.2 2000/02/01 22:32:53 d Exp $");
 
 #include "xmalloc.h"
 #include "ssh.h"
@@ -32,14 +32,14 @@ RCSID("$Id: atomicio.c,v 1.1 1999/12/06 20:15:25 deraadt Exp $");
 /*
  * ensure all of data on socket comes through. f==read || f==write
  */
-int
+ssize_t
 atomicio(f, fd, s, n)
-	int (*f) ();
+	ssize_t (*f) ();
 	int fd;
 	void *s;
 	size_t n;
 {
-	int res, pos = 0;
+	ssize_t res, pos = 0;
 
 	while (n > pos) {
 		res = (f) (fd, s + pos, n - pos);
