@@ -11,7 +11,7 @@
  */
 
 #include "includes.h"
-RCSID("$OpenBSD: sshd.c,v 1.80 2000/01/20 15:19:22 markus Exp $");
+RCSID("$OpenBSD: sshd.c,v 1.90 2000/03/06 20:29:04 markus Exp $");
 
 #include "xmalloc.h"
 #include "rsa.h"
@@ -143,6 +143,27 @@ void do_child(const char *command, struct passwd * pw, const char *term,
 	      const char *display, const char *auth_proto,
 	      const char *auth_data, const char *ttyname);
 
+/*
+ * Remove local Xauthority file.
+ */
+void
+xauthfile_cleanup_proc(void *ignore)
+{
+	debug("xauthfile_cleanup_proc called");
+
+	if (xauthfile != NULL) {
+		char *p;
+		unlink(xauthfile);
+		p = strrchr(xauthfile, '/');
+		if (p != NULL) {
+			*p = '\0';
+			rmdir(xauthfile);
+		}
+		xfree(xauthfile);
+		xauthfile = NULL;
+	}
+}
+
 /*
  * Close all listening sockets
  */
@@ -230,6 +251,7 @@ grace_alarm_handler(int sig)
 char *
 get_authname(int type)
 {
+	static char buf[1024];
 	switch (type) {
 	case SSH_CMSG_AUTH_PASSWORD:
 		return "password";
@@ -248,8 +270,8 @@ get_authname(int type)
 		return "s/key";
 #endif
 	}
-	fatal("get_authname: unknown auth %d: internal error", type);
-	return NULL;
+	snprintf(buf, sizeof buf, "bad-auth-msg-%d", type);
+	return buf;
 }
 
 /*
@@ -873,7 +895,7 @@ main(int ac, char **av)
 
 	/* Cleanup user's local Xauthority file. */
 	if (xauthfile)
-		unlink(xauthfile);
+		xauthfile_cleanup_proc(NULL);
 
 	/* The connection has been terminated. */
 	verbose("Closing connection to %.100s", remote_ip);
@@ -1079,12 +1101,14 @@ do_ssh_kex()
  * DenyUsers or user's primary group is listed in DenyGroups, false will
  * be returned. If AllowUsers isn't empty and user isn't listed there, or
  * if AllowGroups isn't empty and user isn't listed there, false will be
- * returned. Otherwise true is returned.
- * XXX This function should also check if user has a valid shell
+ * returned. 
+ * If the user's shell is not executable, false will be returned.
+ * Otherwise true is returned. 
  */
 static int
 allowed_user(struct passwd * pw)
 {
+	struct stat st;
 	struct group *grp;
 	int i;
 
@@ -1092,7 +1116,11 @@ allowed_user(struct passwd * pw)
 	if (!pw)
 		return 0;
 
-	/* XXX Should check for valid login shell */
+	/* deny if shell does not exists or is not executable */
+	if (stat(pw->pw_shell, &st) != 0)
+		return 0;
+	if (!((st.st_mode & S_IFREG) && (st.st_mode & (S_IXOTH|S_IXUSR|S_IXGRP))))
+		return 0;
 
 	/* Return false if user is listed in DenyUsers */
 	if (options.num_deny_users > 0) {
@@ -1179,6 +1207,7 @@ do_authentication()
 	pw = getpwnam(user);
 	if (!pw || !allowed_user(pw))
 		do_fake_authloop(user);
+	xfree(user);
 
 	/* Take a copy of the returned structure. */
 	memset(&pwcopy, 0, sizeof(pwcopy));
@@ -1197,7 +1226,7 @@ do_authentication()
 	if (getuid() != 0 && pw->pw_uid != getuid())
 		packet_disconnect("Cannot change user when server not running as root.");
 
-	debug("Attempting authentication for %.100s.", user);
+	debug("Attempting authentication for %.100s.", pw->pw_name);
 
 	/* If the user has no password, accept authentication immediately. */
 	if (options.password_authentication &&
@@ -1520,6 +1549,7 @@ do_fake_authloop(char *user)
 		/* Try to send a fake s/key challenge. */
 		if (options.skey_authentication == 1 &&
 		    (skeyinfo = skey_fake_keyinfo(user)) != NULL) {
+			password = NULL;
 			if (type == SSH_CMSG_AUTH_TIS) {
 				packet_start(SSH_SMSG_AUTH_TIS_CHALLENGE);
 				packet_put_string(skeyinfo, strlen(skeyinfo));
@@ -1533,6 +1563,8 @@ do_fake_authloop(char *user)
 			           strncasecmp(password, "s/key", 5) == 0 ) {
 				packet_send_debug(skeyinfo);
 			}
+			if (password != NULL)
+				xfree(password);
 		}
 #endif
 		if (attempt > AUTH_FAIL_MAX)
@@ -1550,20 +1582,35 @@ do_fake_authloop(char *user)
 	abort();
 }
 
+struct pty_cleanup_context {
+	const char *ttyname;
+	int pid;
+};
 
 /*
- * Remove local Xauthority file.
+ * Function to perform cleanup if we get aborted abnormally (e.g., due to a
+ * dropped connection).
  */
-static void
-xauthfile_cleanup_proc(void *ignore)
+void 
+pty_cleanup_proc(void *context)
 {
-	debug("xauthfile_cleanup_proc called");
+	struct pty_cleanup_context *cu = context;
 
-	if (xauthfile != NULL) {
-		unlink(xauthfile);
-		xfree(xauthfile);
-		xauthfile = NULL;
-	}
+	debug("pty_cleanup_proc called");
+
+	/* Record that the user has logged out. */
+	record_logout(cu->pid, cu->ttyname);
+
+	/* Release the pseudo-tty. */
+	pty_release(cu->ttyname);
+}
+
+/* simple cleanup: chown tty slave back to root */
+static void
+pty_release_proc(void *tty)
+{
+	char *ttyname = tty;
+	pty_release(ttyname);
 }
 
 /*
@@ -1577,14 +1624,10 @@ do_authenticated(struct passwd * pw)
 {
 	int type;
 	int compression_level = 0, enable_compression_after_reply = 0;
-	int have_pty = 0, ptyfd = -1, ttyfd = -1, xauthfd = -1;
+	int have_pty = 0, ptyfd = -1, ttyfd = -1;
 	int row, col, xpixel, ypixel, screen;
 	char ttyname[64];
-	char *command, *term = NULL, *display = NULL, *proto = NULL,
-	*data = NULL;
-	struct group *grp;
-	gid_t tty_gid;
-	mode_t tty_mode;
+	char *command, *term = NULL, *display = NULL, *proto = NULL, *data = NULL;
 	int n_bytes;
 
 	/*
@@ -1600,7 +1643,8 @@ do_authenticated(struct passwd * pw)
 	 * by the client telling us, so we can equally well trust the client
 	 * not to request anything bogus.)
 	 */
-	channel_permit_all_opens();
+	if (!no_port_forwarding_flag)
+		channel_permit_all_opens();
 
 	/*
 	 * We stay in this loop until the client requests to execute a shell
@@ -1642,33 +1686,20 @@ do_authenticated(struct passwd * pw)
 				error("Failed to allocate pty.");
 				goto fail;
 			}
-			/* Determine the group to make the owner of the tty. */
-			grp = getgrnam("tty");
-			if (grp) {
-				tty_gid = grp->gr_gid;
-				tty_mode = S_IRUSR | S_IWUSR | S_IWGRP;
-			} else {
-				tty_gid = pw->pw_gid;
-				tty_mode = S_IRUSR | S_IWUSR | S_IWGRP | S_IWOTH;
-			}
-
-			/* Change ownership of the tty. */
-			if (chown(ttyname, pw->pw_uid, tty_gid) < 0)
-				fatal("chown(%.100s, %d, %d) failed: %.100s",
-				      ttyname, pw->pw_uid, tty_gid, strerror(errno));
-			if (chmod(ttyname, tty_mode) < 0)
-				fatal("chmod(%.100s, 0%o) failed: %.100s",
-				      ttyname, tty_mode, strerror(errno));
+			fatal_add_cleanup(pty_release_proc, (void *)ttyname);
+			pty_setowner(pw, ttyname);
 
 			/* Get TERM from the packet.  Note that the value may be of arbitrary length. */
 			term = packet_get_string(&dlen);
 			packet_integrity_check(dlen, strlen(term), type);
-			/* packet_integrity_check(plen, 4 + dlen + 4*4 + n_bytes, type); */
+
 			/* Remaining bytes */
 			n_bytes = plen - (4 + dlen + 4 * 4);
 
-			if (strcmp(term, "") == 0)
+			if (strcmp(term, "") == 0) {
+				xfree(term);
 				term = NULL;
+			}
 
 			/* Get window size from the packet. */
 			row = packet_get_int();
@@ -1714,16 +1745,20 @@ do_authenticated(struct passwd * pw)
 
 			/* Setup to always have a local .Xauthority. */
 			xauthfile = xmalloc(MAXPATHLEN);
-			snprintf(xauthfile, MAXPATHLEN, "/tmp/XauthXXXXXX");
-
-			if ((xauthfd = mkstemp(xauthfile)) != -1) {
-				fchown(xauthfd, pw->pw_uid, pw->pw_gid);
-				close(xauthfd);
-				fatal_add_cleanup(xauthfile_cleanup_proc, NULL);
-			} else {
+			strlcpy(xauthfile, "/tmp/ssh-XXXXXXXX", MAXPATHLEN);
+			temporarily_use_uid(pw->pw_uid);
+			if (mkdtemp(xauthfile) == NULL) {
+				restore_uid();
+				error("private X11 dir: mkdtemp %s failed: %s",
+				    xauthfile, strerror(errno));
 				xfree(xauthfile);
 				xauthfile = NULL;
+				goto fail;
 			}
+			strlcat(xauthfile, "/cookies", MAXPATHLEN);
+			open(xauthfile, O_RDWR|O_CREAT|O_EXCL, 0600);
+			restore_uid();
+			fatal_add_cleanup(xauthfile_cleanup_proc, NULL);
 			break;
 #else /* XAUTH_PATH */
 			packet_send_debug("No xauth program; cannot forward with spoofing.");
@@ -1938,29 +1973,6 @@ do_exec_no_pty(const char *command, struct passwd * pw,
 #endif /* USE_PIPES */
 }
 
-struct pty_cleanup_context {
-	const char *ttyname;
-	int pid;
-};
-
-/*
- * Function to perform cleanup if we get aborted abnormally (e.g., due to a
- * dropped connection).
- */
-void 
-pty_cleanup_proc(void *context)
-{
-	struct pty_cleanup_context *cu = context;
-
-	debug("pty_cleanup_proc called");
-
-	/* Record that the user has logged out. */
-	record_logout(cu->pid, cu->ttyname);
-
-	/* Release the pseudo-tty. */
-	pty_release(cu->ttyname);
-}
-
 /*
  * This is called to fork and execute a command when we have a tty.  This
  * will call do_child from the child, and server_loop from the parent after
@@ -1974,6 +1986,7 @@ do_exec_pty(const char *command, int ptyfd, int ttyfd,
 	    const char *auth_data)
 {
 	int pid, fdout;
+	int ptymaster;
 	const char *hostname;
 	time_t last_login_time;
 	char buf[100], *time_string;
@@ -2096,15 +2109,6 @@ do_exec_pty(const char *command, int ptyfd, int ttyfd,
 	/* Parent.  Close the slave side of the pseudo tty. */
 	close(ttyfd);
 
-	/*
-	 * Create another descriptor of the pty master side for use as the
-	 * standard input.  We could use the original descriptor, but this
-	 * simplifies code in server_loop.  The descriptor is bidirectional.
-	 */
-	fdout = dup(ptyfd);
-	if (fdout < 0)
-		packet_disconnect("dup failed: %.100s", strerror(errno));
-
 	/*
 	 * Add a cleanup function to clear the utmp entry and record logout
 	 * time in case we call fatal() (e.g., the connection gets closed).
@@ -2112,10 +2116,25 @@ do_exec_pty(const char *command, int ptyfd, int ttyfd,
 	cleanup_context.pid = pid;
 	cleanup_context.ttyname = ttyname;
 	fatal_add_cleanup(pty_cleanup_proc, (void *) &cleanup_context);
+	fatal_remove_cleanup(pty_release_proc, (void *) ttyname);
+
+	/*
+	 * Create another descriptor of the pty master side for use as the
+	 * standard input.  We could use the original descriptor, but this
+	 * simplifies code in server_loop.  The descriptor is bidirectional.
+	 */
+	fdout = dup(ptyfd);
+	if (fdout < 0)
+		packet_disconnect("dup #1 failed: %.100s", strerror(errno));
+
+	/* we keep a reference to the pty master */
+	ptymaster = dup(ptyfd);
+	if (ptymaster < 0)
+		packet_disconnect("dup #2 failed: %.100s", strerror(errno));
 
 	/* Enter interactive session. */
 	server_loop(pid, ptyfd, fdout, -1);
-	/* server_loop has not closed ptyfd and fdout. */
+	/* server_loop _has_ closed ptyfd and fdout. */
 
 	/* Cancel the cleanup function. */
 	fatal_remove_cleanup(pty_cleanup_proc, (void *) &cleanup_context);
@@ -2131,8 +2150,8 @@ do_exec_pty(const char *command, int ptyfd, int ttyfd,
 	 * the pty cleanup, so that another process doesn't get this pty
 	 * while we're still cleaning up.
 	 */
-	close(ptyfd);
-	close(fdout);
+	if (close(ptymaster) < 0)
+		error("close(ptymaster): %s", strerror(errno));
 }
 
 /*
@@ -2443,7 +2462,7 @@ do_child(const char *command, struct passwd * pw, const char *term,
 				f = popen(XAUTH_PATH " -q -", "w");
 				if (f) {
 					fprintf(f, "add %s %s %s\n", display, auth_proto, auth_data);
-					fclose(f);
+					pclose(f);
 				} else
 					fprintf(stderr, "Could not run %s -q -\n", XAUTH_PATH);
 			}
